@@ -26,6 +26,22 @@
 
 """PyCOM root package."""
 
+# Aliases from ZeroJSON
+
+from zerojson import (
+    RemoteError,            # Exception on remote side
+    MethodNotFound,         # Remote method not found
+    ServiceNotFound,        # Remote service not found
+    BadRequest,             # Client issued invalid request
+    AccessDenied,           # Access denied to this function
+    ServiceNotAvailable,    # Remote service not available
+    SessionExpired,         # Client session has expired
+    create_task,            # Create repeatable task
+    ioloop,                 # IOLoop instance for a service
+    )
+
+# Our own imports
+
 from .constants import (
     __version__,        # Python standard version definition
     __version_info__,   # Python standard version definition
@@ -35,23 +51,11 @@ from .utils import (
     logger,             # Python logger for PyCOM
     )
 
-from .exceptions import (
-    Error,                  # Base exception
-    MethodNotFound,         # Remote method not found
-    ServiceNotFound,        # Remote service not found
-    RemoteError,            # Exception on remote side
-    BadRequest,             # Client issued invalid request
-    AccessDenied,           # Access denied to this function
-    ServiceNotAvailable,    # Remote service not available
-    ConfigurationError,     # Error in configuration
-    )
-
 from .base import (
-    BaseComponent,      # ABC for components
-    Request,            # Request structure
-    Response,           # Response structure
+    BaseComponent,          # ABC for components
+    ConfigurationError,     # Error in configuration
     configuration,      # Dictionary with configuration (do not alter directly)
-    configure,          # Safe way to alter configuration
+    configure,              # Safe way to alter configuration
     )
 
 from .helpers import (
@@ -60,16 +64,8 @@ from .helpers import (
     method,             # Decorator for methods
     )
 
-from .zerojson import (
-    RemoteComponent,    # Remote component implementation
-    create_task,        # Create repeatable task
-    ioloop,             # IOLoop instance for a service
-    )
-
 from .nsclient import (
     Context,            # Client context object
-    locate,             # Locate service on network
-    nameserver,         # Function to locate nameserver
     )
 
 from .launcher import (
