@@ -40,7 +40,7 @@ from pycom.ext import checks
 _STRING_AND_NONE = string_types + (None.__class__,)
 
 
-@pycom.interface(pycom.constants.NS_INTERFACE)
+@pycom.interface(pycom.constants.NS_INTERFACE, authentication="never")
 class NameServer(pycom.Service):
     """Class representing nameserver for PyCOM."""
 
@@ -71,7 +71,8 @@ class NameServer(pycom.Service):
         else:
             first_time = True
 
-        def _expire():  # pylint: disable-msg=C0111
+        def _expire():
+            """Utility to drop a service on timeout."""
             pycom.logger().info("Unregistering service '%s' due to timeout",
                 service)
             self._drop_service(service)
