@@ -53,22 +53,26 @@ class BaseComponent(object):
         self.name = interface
         self.context = context
 
-    def invoke(self, method_name, args=None,
-            timeout=None):
+    def invoke(self, method_name, args=None, timeout=None, attachment=None):
         """Calls *method_name* on a service, passing *args* as argument.
 
         *args* should be anything JSON-serializable.
         *timeout* can be used to prevent *invoke* from hanging forever.
         *timeout* is in milliseconds and defaults to 5000.
 
+        *attachment* is any binary of type ``bytes``.
+
         Raises :exc:`pycom.MethodNotFound` if service doesn't provide
         method *method_name*. Raises :exc:`pycom.ServiceNotAvailable`
         if service cannot be reached during request.
 
-        Returns call result as a JSON-serializable entity.
+        Returns call result as a JSON-serializable entity if there is no
+        attachments. If an attachment is present, result is a tuple
+        with the first item being JSON result, the second being an attachment.
 
         """
-        request = zerojson.Request(self.name, method_name, args=args)
+        request = zerojson.Request(self.name, method_name, args=args,
+            attachment=attachment)
 
         if self.context is not None:
             for hook in self.context.prehooks:
@@ -80,7 +84,10 @@ class BaseComponent(object):
             for hook in self.context.posthooks:
                 response = hook(self, response)
 
-        return response.result
+        if response.attachment is not None:
+            return response.result, response.attachment
+        else:
+            return response.result
 
     def invoke_with_request(self, request,
             timeout=None):  # pylint: disable-msg=W0613
@@ -137,6 +144,12 @@ def configure(*args, **kwargs):
     configuration.update(kwargs)
 
 
+# Protected
+
+
+interface_registry = {}  #: Local interfaces registry.
+
+
 # Private
 
 
