@@ -30,7 +30,7 @@
 
 # Versions
 
-__version_info__ = (0, 5, 1, "")
+__version_info__ = (0, 6, 0, "")
 __version__ = "%d.%d.%d%s" % __version_info__
 
 # Standard stuff
@@ -46,6 +46,13 @@ NS_METHOD_REGISTER = "register"
 NS_METHOD_LIST_SERVICES = "list_services"
 NS_SERVICE_TIMEOUT = 60000
 
+# Authenticator constants
+
+AUTH_INTERFACE = "org.pycom.authenticator"
+AUTH_METHOD_AUTHENTICATE = "authenticate"
+AUTH_METHOD_VALIDATE = "validate"
+AUTH_EXTENSION = "org.pycom.auth"
+
 # Other constants
 
 NS_NAME_LOCAL = "local"
