@@ -77,7 +77,8 @@ def check_argument(name, valid_types=None, validator=None,
     assert valid_types is None or isinstance(valid_types, (tuple, type))
     assert validator is None or callable(validator)
 
-    def _checker(iface, method, request):  # pylint: disable-msg=C0111
+    def _checker(iface, method, request):
+        """This prehook checks incoming data."""
         if name not in request.args:
             # We do NOT check presence!
             return request
@@ -106,7 +107,8 @@ def check_argument(name, valid_types=None, validator=None,
 
         return request
 
-    def _wrapper(wrapped):  # pylint: disable-msg=C0111
+    def _wrapper(wrapped):
+        """This wrapper is called on method creation by decorator."""
         wrapped.__method__.prehooks.append(_checker)
         return wrapped
 
