@@ -31,7 +31,7 @@ from six import iteritems, string_types
 from . import constants, interfaces
 
 
-def interface(name):
+def interface(name, **kwargs):
     """Decorator for interface *name*.
 
     Example::
@@ -44,18 +44,31 @@ def interface(name):
     Deriving from :class:`pycom.Service` is completely optional,
     but this base class provides support for introspecting your methods.
 
+    Interfaces are stateless by default, you can make them steteful by
+    using ``stateful`` argument::
+
+        import pycom
+
+        @pycom.interface("com.example.interface.foo", stateful=True)
+        class ClassProvidingInterface(pycom.Service): pass
+
+    Argument ``authentication`` overrides global authentication policy
+    and can take two string values: ``"required"`` and ``"never"``.
+
     """
-    def _wrapper(wrapped):  # pylint: disable-msg=C0111
+    def _wrapper(wrapped):
+        """This callable temporary wraps method."""
         if not isinstance(getattr(wrapped, "introspection_data", {}), dict):
             raise TypeError("introspection_data should be a dict, if present")
 
-        wrapped.__interface__ = interfaces.Interface(wrapped, name)
+        wrapped.__interface__ = interfaces.Interface(wrapped, name, **kwargs)
         return wrapped
 
     return _wrapper
 
 
-def method(name=None, body=None, method_factory=None, results=()):
+def method(name=None, body=None, method_factory=None, results=(),
+        attachments=(None, None)):
     """Decorator for method *name*.
 
     Method should take at least one positional argument: *request*,
@@ -124,6 +137,33 @@ def method(name=None, body=None, method_factory=None, results=()):
             def method_name(self, request, arg1, arg2, arg3=None):
                 return arg2, arg3
 
+    If you are going to use introspection (and e.g.
+    :class:`pycom.ProxyComponent`) with the method, you may also fill
+    *attachments* argument with a tuple of 2 elements: names for
+    incoming and outgoing attachment parameters or ``None``'s::
+
+        import pycom
+
+        @pycom.interface("com.example.interface.foo")
+        class ClassProvidingInterface(pycom.Service):
+
+            @pycom.method(attachments=("blob", None))
+            def method_name(self, request, blob):
+                return blob.decode("utf-8")
+
+    Note that incoming attachment turned into an argument to the method.
+    For outgoing attachment you should add outgoing attachment name to
+    results list and return the attachment in a result tuple::
+
+        import pycom
+
+        @pycom.interface("com.example.interface.foo")
+        class ClassProvidingInterface(pycom.Service):
+
+            @pycom.method(attachments=(None, "blob"), results=("orig", "blob"))
+            def method_name(self, request, string):
+                return string, string.encode("utf-8")
+
     Finally, you can explicitly state remote method name::
 
         import pycom
@@ -134,17 +174,24 @@ def method(name=None, body=None, method_factory=None, results=()):
             @pycom.method("method_name")
             def method_impl(self, request): print request.args
 
+    If interface is stateful (see :func:`pycom.interface`), ``request``
+    will have ``session`` attribute with dictionary.
+    You can store arbitrary data there and
+    it will be persisted across requests.
+
     """
     method_factory = method_factory or interfaces.Method
     if body is None:
         if name is not None and not isinstance(name, string_types):
             name, wrapped = name.__name__, name
-            wrapped.__method__ = method_factory(wrapped, name, results)
+            wrapped.__method__ = method_factory(wrapped, name)
             return wrapped
 
-        def _wrapper(wrapped):  # pylint: disable-msg=C0111
+        def _wrapper(wrapped):
+            """This callable temporary wraps method."""
             new_name = name if name is not None else wrapped.__name__
-            wrapped.__method__ = method_factory(wrapped, new_name, results)
+            wrapped.__method__ = method_factory(wrapped, new_name, results,
+                attachments)
             return wrapped
 
         return _wrapper
@@ -185,13 +232,21 @@ class Service(object):
               "required_arguments": [],
               "optional_arguments": [],
               "results": ["name", "methods", "docstring", "data"],
-              "docstring": "Provides introspection information for a service."
+              "docstring": "Provides introspection information for a service.",
+              "attachments": {
+                "incoming": None,
+                "outgoing": None
+              }
             },
             "method_name": {
               "required_arguments": ["arg1", "arg2"],
               "optional_arguments": ["arg3"],
               "results": ["res1", "res2"],
-              "docstring": "My Demo Method"
+              "docstring": "My Demo Method",
+              "attachments": {
+                "incoming": None,
+                "outgoing": None
+              }
             }
           },
           "docstring": "My Demo Service",
@@ -216,7 +271,11 @@ class Service(object):
                 "required_arguments": list(info.required_arguments),
                 "optional_arguments": list(info.optional_arguments),
                 "results": list(info.results),
-                "docstring": info.wrapped.__doc__
+                "docstring": info.wrapped.__doc__,
+                "attachments": dict(
+                    incoming=info.attachments[0],
+                    outgoing=info.attachments[1]
+                )
             })
             for name, info in iteritems(self.__interface__.methods)
         )
