@@ -32,10 +32,7 @@ from six import itervalues
 
 import zerojson
 
-from . import base
-
-
-registry = {}
+from . import base, constants, proxy
 
 
 class Interface(base.BaseComponent):
@@ -56,20 +53,31 @@ class Interface(base.BaseComponent):
        Dictionary of methods for this class, keys being method names,
        value being objects of type :class:`pycom.interfaces.Method`.
 
+    .. attribute:: stateful
+
+       Boolean value, whether this interface is stateful or not.
+
+    .. attribute:: authentication
+
+       Authentication policy - see :func:`pycom.interface`.
+
     """
 
-    __slots__ = ("instance", "methods")
+    __slots__ = ("instance", "methods", "stateful", "authentication")
 
-    def __init__(self, wrapped, name):
+    def __init__(self, wrapped, name, stateful=False, authentication=None):
         """C-tor. Never call it directly, use pycom.interface decorator."""
         super(Interface, self).__init__(name)
 
-        if name in registry and not isinstance(registry[name], wrapped):
+        if name in base.interface_registry and not \
+                isinstance(base.interface_registry[name], wrapped):
             raise RuntimeError("Interface %s is already registered "
-                "for object %s" % (name, repr(registry[name])))
+                "for object %s" % (name, repr(base.interface_registry[name])))
 
         self.instance = wrapped()
         self.methods = {}
+        self.stateful = stateful
+        self.authentication = authentication
 
         for key in dir(wrapped):
             method_info = getattr(getattr(wrapped, key), "__method__", None)
@@ -81,7 +89,7 @@ class Interface(base.BaseComponent):
         for method_info in itervalues(self.methods):
             method_info.post_configure(self)
 
-        registry[name] = self.instance
+        base.interface_registry[name] = self.instance
 
     def register_method(self, method_info):
         """Register method with descriptor *method_info*
@@ -189,18 +197,25 @@ class Method(object):
         If present, output will be checked, resulting tuple will be unpacked,
         and JSON object will be sent back.
 
+    .. attribute:: attachments
+
+        A tuple of 2 strings: names for incoming and outgoing attachment
+        parameters. Mostly used in introspection and
+        :class:`pycom.ProxyComponent`. Defaults to ``(None, None)``.
+
     """
 
     __slots__ = ("wrapped", "name", "prehooks", "posthooks",
-        "required_arguments", "optional_arguments", "results")
+        "required_arguments", "optional_arguments", "results", "attachments")
 
-    def __init__(self, wrapped, name, results=()):
+    def __init__(self, wrapped, name, results=(), attachments=(None, None)):
         """C-tor. Never call it directly, use pycom.method decorator."""
         self.wrapped = wrapped
         self.name = name
         self.prehooks = []
         self.posthooks = []
         self.results = results
+        self.attachments = attachments
 
         info = inspect.getargspec(wrapped)
         if info.defaults:
@@ -221,6 +236,25 @@ class Method(object):
         Returns :class:`zerojson.Response` object.
 
         """
+        # Client context
+
+        token = request.extensions.get(constants.AUTH_EXTENSION)
+
+        def _context_builder():
+            """Utility to generate new context."""
+            ctx = proxy.ProxyContext()
+            if token is not None:
+                ctx.authenticate(token=token)
+            return ctx
+
+        request.context = _context_builder
+
+        if _auth_policy(iface) == "required":
+            if request.context().user_info() is None:
+                raise zerojson.AccessDenied("Authentication required")
+
+        # Prepare arguments
+
         if self.required_arguments or self.optional_arguments:
             if request.args is None:
                 request.args = {}
@@ -228,20 +262,32 @@ class Method(object):
                 raise zerojson.BadRequest("Expected dictionary as argument "
                     "for %s.%s" % (iface.name, self.name))
 
+        # Run prehooks
+
         for hook in self.prehooks:
             request = hook(iface, self, request)
 
-        args = tuple(_fetch_argument(request.args, name, iface.name, self.name)
+        # Finish processing arguments
+
+        args = tuple(_fetch_argument(request, name, iface.name, self.name,
+                self.attachments[0])
             for name in self.required_arguments)
         kwargs = dict((name, request.args[name])
             for name in self.optional_arguments
             if name in request.args)
 
+        # Invoke method
+
         response = self.wrapped(iface.instance, request, *args, **kwargs)
+
+        # Future objects support
+
         if isinstance(response, zerojson.Future):
             response.add_callback(self._after_call, iface)
             return response
 
+        # Response post-processing
+
         if not isinstance(response, zerojson.Response):
             response = request.response(response)
 
@@ -251,9 +297,10 @@ class Method(object):
 
     def _after_call(self, iface, response):
         """Routine to run after call."""
-        if len(self.results) == 1:
-            response.result = {self.results[0]: response.result}
-        elif len(self.results) > 1:
+        if self.results:
+            if len(self.results) == 1:
+                response.result = (response.result,)
+
             if not isinstance(response.result, tuple) \
                     or len(response.result) != len(self.results):
                 raise RuntimeError(
@@ -261,18 +308,33 @@ class Method(object):
                     (iface.name, self.name, len(self.results)))
             response.result = dict(zip(self.results, response.result))
 
+            att_name = self.attachments[1]
+            if att_name in response.result:
+                response.attachment = response.result[att_name]
+                del response.result[att_name]
+
         for hook in self.posthooks:
             response = hook(iface, self, response)
 
         return response
 
+
 # Private
 
 
-def _fetch_argument(args, name, method_name, iface_name):
+def _auth_policy(iface):
+    """Get authentication policy for an interfaces."""
+    return iface.authentication or \
+        base.configuration.get("authentication", {}).get("policy")
+
+
+def _fetch_argument(request, name, method_name, iface_name, att_name):
     """Fetch given argument from args."""
+    if name == att_name:
+        return request.attachment
+
     try:
-        return args[name]
+        return request.args[name]
     except KeyError:
         raise zerojson.BadRequest("Argument '%s' is required for %s.%s" %
             (name, method_name, iface_name))
