@@ -27,16 +27,28 @@
 """Proxy Context API."""
 
 import collections
+import functools
 
 from . import constants, nsclient
 
 
 class ProxyContext(object):
-    """Context object with handy access to methods."""
+    """Context object with handy access to methods.
+
+    Can also be used to wrap a real :class:`pycom.Context` object, e.g.::
+
+        ctx = pycom.Context()
+        # ...
+        proxy_ctx = pycom.ProxyContext(ctx)
+
+    """
 
     def __init__(self, *args, **kwargs):
         """Constructor."""
-        self._wrapped = nsclient.Context(*args, **kwargs)
+        if args and isinstance(args[0], nsclient.Context):
+            self._wrapped = args[0]
+        else:
+            self._wrapped = nsclient.Context(*args, **kwargs)
 
     def __getattr__(self, name):
         """Pass calls to a real context."""
@@ -88,6 +100,10 @@ class ProxyComponent(object):
 
     Methods with multiple results return `namedtuple`.
 
+    Be careful with attachments: they will NOT work unless properly declared
+    using ``attachments`` and ``results`` arguments of :func:`pycom.method`.
+    Both attachments are dialed with just like any other argument.
+
     """
 
     def __init__(self, wrapped):
@@ -111,36 +127,52 @@ class ProxyComponent(object):
         except KeyError:
             raise AttributeError("Method '%s' not found" % name)
 
-        results = method_info["results"]
-        if len(results) > 1:
-            result_type = collections.namedtuple("ResultType",
-                " ".join(method_info["results"]))
+        result = functools.partial(_proxy_attribute, name, self._wrapped,
+            method_info["required_arguments"], method_info["results"],
+            method_info["attachments"])
+        result.__name__ = name
+        result.__doc__ = method_info.get("docstring") or \
+            "Remote method '%s.%s'." % (self._wrapped.name, name)
+        return result
 
-        required_argument = method_info["required_arguments"]
-        del method_info
 
-        def _call(*args, **kwargs):  # pylint: disable-msg=C0111
-            if len(args) > len(required_argument):
-                raise TypeError("Too many positional arguments, expected %d" %
-                    len(required_argument))
+# Private
 
-            for arg, value in zip(required_argument, args):
-                if arg in kwargs:
-                    raise TypeError("Argument '%s' got multiple values" % arg)
-                kwargs[arg] = value
 
-            for arg in required_argument:
-                if arg not in kwargs:
-                    raise TypeError("Argument '%s' is required" % arg)
+def _proxy_attribute(name, wrapped, required_arguments, results, attachments,
+        *args, **kwargs):
+    """Callable that handles call to remote method."""
+    if len(results) > 1:
+        result_type = collections.namedtuple("ResultType", " ".join(results))
 
-            result = self._wrapped.invoke(name, args=kwargs)
+    if len(args) > len(required_arguments):
+        raise TypeError("Too many positional arguments, expected %d" %
+            len(required_arguments))
 
-            if len(results) > 1:
-                return result_type._make(  # pylint: disable-msg=W0212
-                    result[item] for item in results)
-            elif len(results) == 1:
-                return result[results[0]]
-            else:
-                return result
+    for arg, value in zip(required_arguments, args):
+        if arg in kwargs:
+            raise TypeError("Argument '%s' got multiple values" % arg)
+        kwargs[arg] = value
 
-        return _call
+    for arg in required_arguments:
+        if arg not in kwargs:
+            raise TypeError("Argument '%s' is required" % arg)
+
+    try:
+        attachment = kwargs[attachments["incoming"]]
+    except KeyError:
+        attachment = None
+
+    result = wrapped.invoke(name, args=kwargs, attachment=attachment)
+
+    if isinstance(result, tuple) and attachments["outgoing"] is not None:
+        result, attachment = result
+        result[attachments["outgoing"]] = attachment
+
+    if len(results) > 1:
+        return result_type._make(  # pylint: disable-msg=W0212
+            result[item] for item in results)
+    elif len(results) == 1:
+        return result[results[0]]
+    else:
+        return result
