@@ -34,10 +34,9 @@ import pycom
 from pycom import constants
 from pycom.apps import nameserver
 
-from zerojson.testing import unittest, Replacer
+from zerojson.testing import unittest, replacer
 from zerojson.testing import OrderedDict, OrderedDictWithSet
 
-
 class BaseTest(object):
 
     def setUp(self):
@@ -47,11 +46,19 @@ class BaseTest(object):
         self.service._registry = OrderedDictWithSet()
 
 
-class TestStat(BaseTest, unittest.TestCase):
+class TestCommon(BaseTest, unittest.TestCase):
 
     def testStat(self):
         self.assertEqual(self.component.invoke(constants.NS_METHOD_STAT), {})
 
+    def testNoAuth(self):
+        old_auth = pycom.configuration.get("authentication", {})
+        pycom.configure(authentication=dict(policy="required"))
+        try:
+            self.component.invoke(constants.NS_METHOD_STAT)
+        finally:
+            pycom.configuration["authentication"] = old_auth
+
 
 class TestRegister(BaseTest, unittest.TestCase):
 
@@ -202,7 +209,7 @@ class TestTimeout(BaseTest, unittest.TestCase):
         pycom.ioloop().add_timeout(datetime.timedelta(milliseconds=100),
             _check)
 
-        with Replacer(constants, "NS_SERVICE_TIMEOUT", 10):
+        with replacer(constants, "NS_SERVICE_TIMEOUT", 10):
             self.component.invoke(constants.NS_METHOD_REGISTER,
                 args={
                     "interfaces": ["test_apps_nameserver.testExpire"],
