@@ -31,18 +31,23 @@ import re
 
 import pycom
 import zerojson
-from pycom import base, constants, interfaces
+from pycom import base, constants, interfaces, nsclient
 
-from zerojson.testing import unittest
+from zerojson.testing import unittest, replacer
 
 class BaseTest(object):
 
     def setUp(self):
-        self.registry_bak = interfaces.registry.copy()
-        interfaces.registry.clear()
+        self.registry_bak = base.interface_registry.copy()
+        base.interface_registry.clear()
+        self.conf_bak = base.configuration.copy()
+        base.configuration.clear()
 
     def tearDown(self):
-        interfaces.registry = self.registry_bak
+        base.configuration.clear()
+        base.configuration.update(self.conf_bak)
+        base.interface_registry.clear()
+        base.interface_registry.update(self.registry_bak)
 
 class DecoratorsTest(BaseTest, unittest.TestCase):
 
@@ -52,7 +57,7 @@ class DecoratorsTest(BaseTest, unittest.TestCase):
         @pycom.interface(iface)
         class EmptyInterface(object): pass
 
-        self.assertIn(iface, interfaces.registry)
+        self.assertIn(iface, base.interface_registry)
         self.assertIsInstance(EmptyInterface.__interface__, interfaces.Interface)
         self.assertIsInstance(EmptyInterface().__interface__, interfaces.Interface)
         self.assertEqual(EmptyInterface.__interface__.name, iface)
@@ -221,6 +226,47 @@ class LocalComponentTest(BaseTest, unittest.TestCase):
         self.assertIs(component.invoke("method1", args=testObject1),
             testObject2)
 
+    def testContextPresent(self):
+        iface = "test.test_components.LocalComponentTest.testContextPresent"
+
+        @pycom.interface(iface)
+        class InterfaceWithMethods(object):
+
+            @pycom.method
+            def method1(self, request):
+                assert isinstance(request.context(), pycom.ProxyContext)
+                return True
+
+        component = InterfaceWithMethods().__interface__
+        self.assertTrue(component.invoke("method1"))
+
+    def testIncomingAttachment(self):
+        iface = "test.test_components.LocalComponentTest.testIncomingAttachment"
+
+        @pycom.interface(iface)
+        class InterfaceWithMethods(object):
+
+            @pycom.method
+            def method1(self, request):
+                return request.attachment
+
+        component = InterfaceWithMethods().__interface__
+        self.assertEqual(component.invoke("method1", attachment=b"Hello"),
+            b"Hello")
+
+    def testOutgoingAttachment(self):
+        iface = "test.test_components.LocalComponentTest.testOutgoingAttachment"
+
+        @pycom.interface(iface)
+        class InterfaceWithMethods(object):
+
+            @pycom.method
+            def method1(self, request):
+                return request.response(42, attachment=b"Bye")
+
+        component = InterfaceWithMethods().__interface__
+        self.assertEqual(component.invoke("method1"), (42, b"Bye"))
+
     def testCallWithResponse(self):
         iface = "test.test_components.LocalComponentTest.testCallWithResponse"
         meth1 = "method1"
@@ -437,6 +483,77 @@ class ArgumentsTest(BaseTest, unittest.TestCase):
             42)
 
 
+class TestAttachmentsInfo(BaseTest, unittest.TestCase):
+
+    def testNo(self):
+        iface = "test.test_interfaces.TestAttachmentsInfo.testNo"
+
+        @pycom.interface(iface)
+        class SimpleInterface(object):
+
+            @pycom.method
+            def method1(self, request): pass  # pragma: no cover
+
+        component = SimpleInterface.__interface__
+        method = component.methods["method1"]
+        self.assertEqual(method.attachments, (None, None))
+
+    def testBoth(self):
+        iface = "test.test_interfaces.TestAttachmentsInfo.testBoth"
+
+        @pycom.interface(iface)
+        class SimpleInterface(object):
+
+            @pycom.method(attachments=("inc", "out"))
+            def method1(self, request): pass  # pragma: no cover
+
+        component = SimpleInterface.__interface__
+        method = component.methods["method1"]
+        self.assertEqual(method.attachments, ("inc", "out"))
+
+    def testCallIncoming(self):
+        iface = "test.test_interfaces.TestAttachmentsInfo.testCallIncoming"
+
+        @pycom.interface(iface)
+        class SimpleInterface(object):
+
+            @pycom.method(attachments=("inc", None))
+            def method1(self, request, inc):
+                return inc.decode("utf-8")
+
+        component = SimpleInterface.__interface__
+        self.assertEqual(component.invoke("method1", attachment=b"Hello"),
+            "Hello")
+
+    def testCallOutgoing(self):
+        iface = "test.test_interfaces.TestAttachmentsInfo.testCallOutgoing"
+
+        @pycom.interface(iface)
+        class SimpleInterface(object):
+
+            @pycom.method(results=("num", "out"), attachments=(None, "out"))
+            def method1(self, request, num):
+                return num, bytes(str(num).encode())
+
+        component = SimpleInterface.__interface__
+        self.assertEqual(component.invoke("method1", args=dict(num=42)),
+            (dict(num=42), b"42"))
+
+    def testCallOutgoing2(self):
+        iface = "test.test_interfaces.TestAttachmentsInfo.testCallOutgoing2"
+
+        @pycom.interface(iface)
+        class SimpleInterface(object):
+
+            @pycom.method(results=("out",), attachments=(None, "out"))
+            def method1(self, request, num):
+                return bytes(str(num).encode())
+
+        component = SimpleInterface.__interface__
+        self.assertEqual(component.invoke("method1", args=dict(num=42)),
+            (dict(), b"42"))
+
+
 class IntrospectionTest(BaseTest, unittest.TestCase):
 
     maxDiff = None
@@ -468,19 +585,31 @@ class IntrospectionTest(BaseTest, unittest.TestCase):
                         "optional_arguments": [],
                         "results": ["name", "methods", "docstring", "data"],
                         "docstring":
-                            "Provides introspection information for a service."
+                            "Provides introspection information for a service.",
+                        "attachments": {
+                            "incoming": None,
+                            "outgoing": None
+                        }
                     },
                     meth1: {
                         "required_arguments": ["arg1"],
                         "optional_arguments": ["arg2", "arg3"],
                         "results": [],
-                        "docstring": "My Demo Method"
+                        "docstring": "My Demo Method",
+                        "attachments": {
+                            "incoming": None,
+                            "outgoing": None
+                        }
                     },
                     meth2: {
                         "required_arguments": [],
                         "optional_arguments": [],
                         "results": ["res1", "res2"],
-                        "docstring": None
+                        "docstring": None,
+                        "attachments": {
+                            "incoming": None,
+                            "outgoing": None
+                        }
                     }
                 },
                 "docstring": "My Demo Service",
@@ -535,3 +664,121 @@ class IntrospectionTest(BaseTest, unittest.TestCase):
                 introspection_data = 42
 
         self.assertRaises(TypeError, _check)
+
+
+class AuthBaseTest(BaseTest):
+
+    def setUp(self):
+        super(AuthBaseTest, self).setUp()
+
+        @pycom.interface(pycom.constants.AUTH_INTERFACE,
+            authentication="never")
+        class DummyAuthenticator(object):
+
+            @pycom.method(pycom.constants.AUTH_METHOD_AUTHENTICATE,
+                results=("token",))
+            def authenticate(self, request, user, credentials):
+                if user == "valid" and credentials == "password":
+                    return "123456"
+                else:
+                    raise pycom.AccessDenied("invalid user")
+
+            @pycom.method(pycom.constants.AUTH_METHOD_VALIDATE,
+                results=("name", "roles"))
+            def validate(self, request, token):
+                if token == "123456":
+                    return "valid", []
+                else:
+                    raise pycom.AccessDenied("invalid token")
+
+        self.service = DummyAuthenticator.__interface__
+
+        def locate2(name):
+            self.assertEqual(name, pycom.constants.AUTH_INTERFACE)
+            return self.service
+
+        self.locate = locate2
+
+        self.tested = pycom.Context()
+        self.tested.locate = locate2
+
+
+class TestServiceWithAuth(AuthBaseTest, unittest.TestCase):
+
+    def testContextFactory(self):
+        @pycom.interface(pycom.constants.NS_INTERFACE)
+        class Nameserver(pycom.Service):
+
+            @pycom.method
+            def start(self2, request):
+                ctx = request.context()
+                ctx.locate = self.locate
+                return ctx.nameserver().finish()
+
+            @pycom.method
+            def finish(self2, request):
+                return request.extensions[pycom.constants.AUTH_EXTENSION]
+
+        self.tested.authenticate("valid", "password")
+        component = self.tested.nameserver()
+        with replacer(nsclient, "Context", lambda: self.tested):
+            self.assertEqual(component.invoke("start"), "123456")
+
+
+class TestCheckRequiredAuth(AuthBaseTest, unittest.TestCase):
+
+    def testNoPolicy(self):
+        @pycom.interface(pycom.constants.NS_INTERFACE)
+        class Nameserver(pycom.Service):
+
+            @pycom.method
+            def test(self2, request): pass
+
+        component = self.tested.nameserver()
+        self.assertEqual(component.invoke("test"), None)
+
+    def testPolicyRequiredFailure(self):
+        @pycom.interface(pycom.constants.NS_INTERFACE)
+        class Nameserver(pycom.Service):
+
+            @pycom.method
+            def test(self2, request): pass  # pragma: no cover
+
+        pycom.configure(authentication=dict(policy="required"))
+        component = self.tested.nameserver()
+        self.assertRaises(pycom.AccessDenied, component.invoke, "test")
+
+    def testPolicyRequiredFailure2(self):
+        @pycom.interface(pycom.constants.NS_INTERFACE,
+            authentication="required")
+        class Nameserver(pycom.Service):
+
+            @pycom.method
+            def test(self2, request): pass  # pragma: no cover
+
+        component = self.tested.nameserver()
+        self.assertRaises(pycom.AccessDenied, component.invoke, "test")
+
+    def testPolicyNeverOk(self):
+        @pycom.interface(pycom.constants.NS_INTERFACE, authentication="never")
+        class Nameserver(pycom.Service):
+
+            @pycom.method
+            def test(self2, request): pass  # pragma: no cover
+
+        pycom.configure(authentication=dict(policy="required"))
+        component = self.tested.nameserver()
+        self.assertEqual(component.invoke("test"), None)
+
+    def testPolicyRequiredOk(self):
+        @pycom.interface(pycom.constants.NS_INTERFACE)
+        class Nameserver(pycom.Service):
+
+            @pycom.method
+            def test(self2, request): pass
+
+        pycom.configure(authentication=dict(policy="required"))
+        self.tested.authenticate("valid", "password")
+        component = self.tested.nameserver()
+        with replacer(nsclient, "Context", lambda: self.tested):
+            self.assertEqual(component.invoke("test"), None)
