@@ -43,7 +43,7 @@ import zerojson
 from pycom import base, constants, nsclient, utils
 from zerojson import call
 
-from zerojson.testing import unittest, AddressGenerationMixin, Replacer
+from zerojson.testing import unittest, AddressGenerationMixin, replacer
 from .test_nsclient import BaseTest
 
 
@@ -81,8 +81,41 @@ class MainTest(BaseTest, AddressGenerationMixin, unittest.TestCase):
             self.assertDictEqual(component.invoke(meth, [1, 2, 3]),
                 {"input": [1, 2, 3]})
 
+        def _testMethod(request):
+            return {"input": request.args}
+
         self._testService(iface, meth, _testMethod, _check, self)
 
+    def testServiceNoSession(self):
+        iface = "test.test_launcher.MainTest.testService"
+        meth = "test_method"
+
+        def _check(address, self):
+            component = nsclient.RemoteComponent(address, iface)
+            self.assertEqual(component.invoke(meth), None)
+
+        def _testMethod(request):
+            return request.session
+
+        self._testService(iface, meth, _testMethod, _check, self)
+
+    def testServiceWithSession(self):
+        iface = "test.test_launcher.MainTest.testService"
+        meth = "test_method"
+
+        def _check(address, self):
+            component = nsclient.RemoteComponent(address, iface)
+            self.assertEqual(component.invoke(meth, 42), 42)
+            self.assertEqual(component.invoke(meth), 42)
+
+        def _testMethod(request):
+            if request.args:
+                request.session["data"] = request.args
+            return request.session["data"]
+
+        self._testService(iface, meth, _testMethod, _check, self,
+            stateful=True)
+
     def testFuture(self):
         iface = "test.test_launcher.MainTest.testFuture"
         meth = "test_method"
@@ -121,6 +154,8 @@ class MainTest(BaseTest, AddressGenerationMixin, unittest.TestCase):
             self.assertEqual(context.exception.code,
                 zerojson.constants.ERROR_BAD_REQUEST)
 
+        def _testMethod(request): pass  # pragma: no cover
+
         self._testService(iface, meth, _testMethod, _check, self)
 
     def testRecreating(self):
@@ -181,13 +216,13 @@ class MainTest(BaseTest, AddressGenerationMixin, unittest.TestCase):
         threading.Thread(target=_check).start()
 
         pycom.configure(address=address)
-        with Replacer(constants, "NS_SERVICE_TIMEOUT", 70):
+        with replacer(constants, "NS_SERVICE_TIMEOUT", 70):
             self.main()
 
     # Private
 
     def _testService(self, iface, meth, meth_body, call, *args, **kw):
-        @pycom.interface(iface)
+        @pycom.interface(iface, stateful=kw.pop("stateful", False))
         class Service(object):
 
             @pycom.method(meth)
@@ -196,9 +231,12 @@ class MainTest(BaseTest, AddressGenerationMixin, unittest.TestCase):
 
         address = self.genAddress()
 
+        ok = [False]
+
         def _check():
             try:
                 call(address, *args, **kw)
+                ok[0] = True
             finally:
                 pycom.ioloop().add_callback(pycom.ioloop().stop)
 
@@ -207,8 +245,7 @@ class MainTest(BaseTest, AddressGenerationMixin, unittest.TestCase):
         pycom.configure(address=address)
         self.main()
 
-
-_testMethod = lambda request: {"input": request.args}
+        self.assertTrue(ok[0], "Test failed, see error above")
 
 
 class DummyNS(object):
@@ -234,11 +271,11 @@ class ArgsTest(AddressGenerationMixin, unittest.TestCase):
         super(ArgsTest, self).tearDown()
 
     def testFailEmptyArgs(self):
-        with Replacer(sys, "stderr", StringIO()):
+        with replacer(sys, "stderr", StringIO()):
             self.assertRaises(SystemExit, self.main, [])
 
     def testFailEmptyAddress(self):
-        with Replacer(sys, "stderr", StringIO()):
+        with replacer(sys, "stderr", StringIO()):
             self.assertRaises(SystemExit, self.main, ["sys", "-a"])
 
     def testGeneratedService(self):
@@ -248,7 +285,7 @@ class ArgsTest(AddressGenerationMixin, unittest.TestCase):
         self.main(["sys", "-a", address])
 
     def testFailUnknownOption(self):
-        with Replacer(sys, "stderr", StringIO()):
+        with replacer(sys, "stderr", StringIO()):
             self.assertRaises(SystemExit, self.main, ["sys", "-x"])
 
     def testCorrectOptions(self):
@@ -274,6 +311,6 @@ class ArgsTest(AddressGenerationMixin, unittest.TestCase):
     def testUnknownModule(self):
         address = self.genAddress()
 
-        with Replacer(sys, "stderr", StringIO()):
+        with replacer(sys, "stderr", StringIO()):
             self.assertRaises(SystemExit, self.main,
                 ["banana!", "-a", address, "-s", "/test/test_launcher4"])
