@@ -36,8 +36,8 @@ import zerojson
 from pycom import base, constants, interfaces, nsclient
 from zerojson import call, common
 
-from zerojson.testing import unittest, Replacer
-from .test_interfaces import BaseTest as InterfacesBaseTest
+from zerojson.testing import unittest, replacer
+from .test_interfaces import BaseTest as InterfacesBaseTest, AuthBaseTest
 
 
 class BaseTest(InterfacesBaseTest):
@@ -85,7 +85,7 @@ class LocalNSTest(BaseTest, unittest.TestCase):
         ns1 = context.nameserver()
         self.assertIs(ns1, context.nameserver())
 
-        interfaces.registry.clear()
+        base.interface_registry.clear()
         self.assertIs(ns1, context.nameserver())
 
     def testNSThreadSafe(self):
@@ -117,12 +117,19 @@ def fake_container(context, event, addr, event2, handler):
         if not sock.poll(timeout=10):
             continue
 
-        cmd, body = sock.recv_multipart()
+        items = sock.recv_multipart()
+        cmd, body = items[:2]
         assert cmd == zerojson.constants.PROTO_CMD_CALL
 
         request = call_command.request_from_wire(body)
-        sock.send_multipart([zerojson.constants.PROTO_STATUS_SUCCESS,
-            call_command.response_to_wire(handler(request))])
+        request.attachment = items[2] if len(items) > 2 else None
+        if request.method == "send me attachment":
+            sock.send_multipart([zerojson.constants.PROTO_STATUS_SUCCESS,
+                call_command.response_to_wire(request.response({})),
+                b"Welcome"])
+        else:
+            sock.send_multipart([zerojson.constants.PROTO_STATUS_SUCCESS,
+                call_command.response_to_wire(handler(request))])
 
     sock.close()
 
@@ -130,10 +137,12 @@ def fake_container(context, event, addr, event2, handler):
 def ns_handler(request):
     assert request.interface == constants.NS_INTERFACE
     assert request.method in (constants.NS_METHOD_LOCATE,
-        constants.NS_METHOD_STAT)
+        constants.NS_METHOD_STAT, "att-demo")
 
     if request.method == constants.NS_METHOD_STAT:
         return request.response({})
+    elif request.method == "att-demo":
+        return request.response(request.attachment.decode("utf-8"))
     else:
         return zerojson.Response(request,
                 code=zerojson.constants.ERROR_SERVICE_NOT_FOUND)
@@ -180,14 +189,22 @@ class RemoteNSTest(BaseTest, unittest.TestCase):
                     "interface": "org.pycom.test_nsclient.NON-EXISTING"
                 })
 
+    def testIncomingAttachment(self):
+        ns = pycom.Context().nameserver()
+        self.assertEqual(ns.invoke("att-demo", attachment=b"Hello"), "Hello")
+
+    def testOutgoingAttachment(self):
+        ns = pycom.Context().nameserver()
+        self.assertEqual(ns.invoke("send me attachment"), ({}, b"Welcome"))
+
     def testUnknownNS(self):
         context = pycom.Context(nameserver="ipc://pycom/banana!")
-        with Replacer(zerojson.constants, "PROTO_DEFAULT_TIMEOUT", 100):
+        with replacer(zerojson.constants, "PROTO_DEFAULT_TIMEOUT", 100):
             self.assertRaises(zerojson.ServiceNotFound, context.nameserver)
 
     def testUnknownNS2(self):
         context = pycom.Context(nameserver="inproc://pycom/banana!")
-        with Replacer(zerojson.constants, "PROTO_DEFAULT_TIMEOUT", 100):
+        with replacer(zerojson.constants, "PROTO_DEFAULT_TIMEOUT", 100):
             self.assertRaises(zerojson.ServiceNotFound, context.nameserver)
 
     def testLocateError(self):
@@ -389,3 +406,55 @@ class ContextHooksTest(BaseTest, unittest.TestCase):
         ctx.posthooks.append(_hook)
         self.assertEqual(ctx.nameserver().invoke("stat", args=42),
             {"output": 42})
+
+
+class TestAuthTokens(AuthBaseTest, unittest.TestCase):
+
+    def setUp(self):
+        super(TestAuthTokens, self).setUp()
+
+    def testGetValidToken(self):
+        self.assertEqual(self.tested._get_token("valid", "password"),
+            "123456")
+
+    def testGetInvalidToken(self):
+        self.assertRaises(pycom.AccessDenied, self.tested._get_token,
+            "invalid", "password")
+        self.assertRaises(pycom.AccessDenied, self.tested._get_token,
+            "valid", "invalid")
+
+    def testCheckValidToken(self):
+        self.assertEqual(self.tested._check_token("123456"),
+            {"name": "valid", "roles": []})
+
+    def testCheckInvalidToken(self):
+        self.assertRaises(pycom.AccessDenied, self.tested._check_token,
+            "invalid")
+
+
+class TestAuthContextHooks(AuthBaseTest, unittest.TestCase):
+
+    def setUp(self):
+        super(TestAuthContextHooks, self).setUp()
+
+        @pycom.interface(pycom.constants.NS_INTERFACE)
+        class Nameserver(object):
+
+            @pycom.method
+            def demo(self, request):
+                return request.extensions[pycom.constants.AUTH_EXTENSION]
+
+    def testContextAuthenticate(self):
+        ctx = self.tested
+        ctx.authenticate("valid", "password")
+        self.assertEqual(ctx.nameserver().invoke("demo"), "123456")
+        self.assertEqual(ctx.user_info(), {"name": "valid", "roles": []})
+
+    def testAuthenticationByToken(self):
+        ctx = self.tested
+        ctx.authenticate(token="123456")
+        self.assertEqual(ctx.nameserver().invoke("demo"), "123456")
+        self.assertEqual(ctx.user_info(), {"name": "valid", "roles": []})
+
+    def testNonAuthContext(self):
+        self.assertIs(self.tested.user_info(), None)
