@@ -31,7 +31,7 @@ import zerojson
 from pycom import base, constants, nsclient
 
 from .test_interfaces import BaseTest
-from zerojson.testing import unittest, Replacer
+from zerojson.testing import unittest, replacer
 
 
 class DummyNS(object):
@@ -74,18 +74,49 @@ class TestContext(BaseTest, unittest.TestCase):
         self.assertIsInstance(result._wrapped, DummyNS)
 
     def testLocate(self):
-        with Replacer(nsclient, "RemoteComponent", dummy_component(self)):
+        with replacer(nsclient, "RemoteComponent", dummy_component(self)):
             result = self.context.locate("dummy.iface")
             self.assertIsInstance(result, pycom.ProxyComponent)
 
     def testConnect(self):
-        with Replacer(nsclient, "RemoteComponent", dummy_component(self)):
+        with replacer(nsclient, "RemoteComponent", dummy_component(self)):
             result = self.context.connect("proxy://dummy", "dummy.iface")
             self.assertIsInstance(result, pycom.ProxyComponent)
 
 
+class TestContextWrap(TestContext):
+
+    def setUp(self):
+        super(TestContext, self).setUp()
+        orig_context = pycom.Context()
+        self.oldNS = nsclient._pop_cached_ns(ctx=orig_context)
+        nsclient._restore_cached_ns(DummyNS(self), ctx=orig_context)
+
+        self.context = pycom.ProxyContext(orig_context)
+
+
 class TestComponent(BaseTest, unittest.TestCase):
 
+    def testCorrectWrapping(self):
+        iface = "test.test_proxy.TestComponent.testCorrectWrapping"
+
+        @pycom.interface(iface)
+        class SimpleInterface(pycom.Service):
+
+            @pycom.method
+            def meth1(self, request):
+                """DOC."""
+                pass  # pragma: no cover
+
+            @pycom.method
+            def meth2(self, request): pass  # pragma: no cover
+
+        with pycom.ProxyComponent(SimpleInterface.__interface__) as component:
+            self.assertEqual(component.meth1.__name__, "meth1")
+            self.assertEqual(component.meth1.__doc__, "DOC.")
+            self.assertEqual(component.meth2.__doc__,
+                "Remote method '%s.meth2'." % iface)
+
     def testRequiredOnly(self):
         iface = "test.test_proxy.TestComponent.testRequiredOnly"
         meth = "method1"
@@ -166,6 +197,22 @@ class TestComponent(BaseTest, unittest.TestCase):
             self.assertEqual(component.method1(1, 2).y, -1)
 
 
+class TestComponentAttachments(BaseTest, unittest.TestCase):
+
+    def testSimple(self):
+        iface = "test.test_proxy.TestComponentAttachments.testAttachments"
+
+        @pycom.interface(iface)
+        class SimpleInterface(pycom.Service):
+
+            @pycom.method(results=("out",), attachments=("inc", "out"))
+            def method1(self, request, inc):
+                return inc
+
+        with pycom.ProxyComponent(SimpleInterface.__interface__) as component:
+            self.assertEqual(component.method1(inc=b"Test"), b"Test")
+
+
 class ContextHooksTest(BaseTest, unittest.TestCase):
 
     def setUp(self):
