@@ -32,7 +32,7 @@ import sys
 import pycom
 from pycom import base, utils
 
-from zerojson.testing import unittest, Replacer
+from zerojson.testing import unittest, replacer
 
 class TestFindConf(unittest.TestCase):
 
@@ -44,7 +44,5 @@ class TestFindConf(unittest.TestCase):
         def _fake_exists(path):
             return path == expected
 
-        with Replacer(os.path, "exists", _fake_exists):
+        with replacer(os.path, "exists", _fake_exists):
             self.assertEqual(utils.find_conf_file("test1.cfg"), expected)
-
-
