@@ -50,7 +50,8 @@ if __name__ == '__main__':
 
     if coverage is not None:
         coverage = coverage.coverage(branch=True,
-            include="*/pycom/*", omit=["test.py", "zerojson/testing.py"])
+            include=["./pycom/*", "./zerojson/*"],
+            omit=["test.py", "zerojson/testing.py"])
         coverage.erase()
         coverage.start()
 
