@@ -52,7 +52,7 @@ class Future(object):
         """
         self._callbacks.append(functools.partial(func, *args, **kwargs))
 
-    def response(self, result):
+    def response(self, result, attachment=None):
         """Send a result.
 
         This method is thread-safe.
@@ -60,7 +60,8 @@ class Future(object):
         """
         if self._getter is not None:
             raise RuntimeError("Response is already set on %s" % repr(self))
-        self._getter = functools.partial(self.request.response, result)
+        self._getter = functools.partial(self.request.response, result,
+            attachment=attachment)
         self._schedule_callbacks()
 
     def error(self, code, message=None):
