@@ -39,10 +39,12 @@ class CallCommand(common.BaseCommand):
         result = {
             "interface": request.interface,
             "method": request.method,
-            "session_id": session_id,
             "args": request.args
         }
 
+        if session_id is not None:
+            result["session_id"] = session_id
+
         if request.extensions:
             result["extensions"] = request.extensions
 
@@ -50,30 +52,20 @@ class CallCommand(common.BaseCommand):
 
     def request_from_dict(self, data):
         """Get request from Python dict."""
-        session_id = data.get("session_id", None)
-
-        if session_id is None:
-            session = self.session_factory.new()
-            utils.logger().debug("NEW SESSION '%s'", session.session_id)
-        else:
-            try:
-                session = self.session_factory.get(session_id)
-            except KeyError:
-                raise exceptions.SessionExpired("Session '%s' has expired" %
-                    session_id)
-
         try:
             return common.Request(data["interface"], data["method"],
-                session=session, args=data.get("args"),
+                session_id=data.get("session_id"),
+                args=data.get("args"),
                 extensions=data.get("extensions"))
         except KeyError as err:
             raise exceptions.BadRequest("Field %s is required" % err.args[0])
 
     def response_to_dict(self, response):
         """Dump response to Python dict."""
-        result = {
-            "session_id": response.session_id
-        }
+        result = {}
+
+        if response.session_id is not None:
+            result["session_id"] = response.session_id
 
         if not response.code:
             result["result"] = response.result
@@ -93,12 +85,14 @@ class CallCommand(common.BaseCommand):
 
     def real_process_request(self, request, *other_parts):
         """Call to process request."""
-        if other_parts:
+        if len(other_parts) > 1:
             return request.error(constants.ERROR_BAD_REQUEST,
-                message="Expected two parts, got %d" % (len(other_parts) + 2))
+                message="Expected 2-3 parts, got %d" % (len(other_parts) + 2))
+        elif other_parts:
+            request.attachment = other_parts[0]
 
         try:
-            response = self.server.handle_call(request)
+            response = self.server.process(request)
         except exceptions.RemoteError as err:
             return request.error(err.code or constants.ERROR_UNKNOWN,
                 message=str(err))
