@@ -69,15 +69,20 @@ class Client(object):
         """
         self.socket.close(**kwargs)
 
-    def invoke(self, method_name, args=None, timeout=None):
+    def invoke(self, method_name, args=None, timeout=None, attachment=None):
         """Invoke given method with args
         (see :meth:`pycom.BaseComponent.invoke` for details)."""
-        return self.invoke_with_request(
-            common.Request(self.iface, method_name, args=args),
-            timeout=timeout).result
+        response = self.invoke_with_request(
+            common.Request(self.iface, method_name, args=args,
+                attachment=attachment),
+            timeout=timeout)
 
-    def invoke_with_request(self, request,
-            timeout=None):  # pylint: disable-msg=W0613
+        if response.attachment is not None:
+            return response.result, response.attachment
+        else:
+            return response.result
+
+    def invoke_with_request(self, request, timeout=None):
         """Invoke method using given request object.
 
         Returns :class:`zerojson.Response` object.
@@ -87,8 +92,9 @@ class Client(object):
             session_id=self.session_id)
         timeout = timeout or constants.PROTO_DEFAULT_TIMEOUT
 
-        status, message = self._send_message(constants.PROTO_CMD_CALL, message,
-            timeout)
+        result = self._send_message(constants.PROTO_CMD_CALL, message,
+            timeout, attachment=request.attachment)
+        status, message = result[:2]
 
         if status != constants.PROTO_STATUS_SUCCESS:
             # Global protocol failure - world is going to explode!
@@ -108,12 +114,15 @@ class Client(object):
         if response.session_id is not None:
             self.session_id = response.session_id
 
+        if len(result) > 2:
+            response.attachment = result[2]
+
         return response
 
     # Private
 
     def _send_message(self, command, message, timeout, attachment=None):
-        """Send prepared message over wire."""
+        """Send prepared message (a list) over wire."""
         if not self.socket.poll(timeout=timeout, flags=zmq.POLLOUT):
             # Timeout - assume peer is disconnected
             raise exceptions.ServiceNotAvailable("Service with interface '%s' "
