@@ -45,6 +45,10 @@ class Server(object):
 
         Dictionary 'command name' -> 'command implementation'.
 
+    .. attribute:: session_factory
+
+        Factory for :class:`zerojson.Session` objects.
+
     """
 
     def __init__(self, addresses):
@@ -60,6 +64,10 @@ class Server(object):
             constants.PROTO_CMD_CALL: call.CallCommand(server=self)
         }
 
+        self.session_factory = eventloop.ExpiringFactory(common.Session,
+            timeout=constants.PROTO_SESSION_TIMEOUT,
+            message="Session '%s' expired")
+
         self.ioloop = None
 
     def setup(self, ioloop=None):
@@ -70,6 +78,23 @@ class Server(object):
             stream.on_recv(functools.partial(self._message_handler, stream),
                 copy=False)
 
+    def process(self, request):
+        """Process the request.
+
+        Do not override this method, override ``handle_call`` instead.
+
+        """
+        return self.handle_call(self.prepare_call(request))
+
+    def prepare_call(self, request):
+        """Prepare request to be passed to ``handle_call``.
+
+        Returns prepared :class:`zerojson.Request`.
+
+        """
+        request.initialize_session(self.session_factory)
+        return request
+
     def handle_call(self, request):
         """Handle a single call."""
         raise NotImplementedError()  # pragma: no cover
@@ -89,7 +114,7 @@ class Server(object):
             request_parts = iter(request_parts)
             address = [
                 x.bytes
-                for x in itertools.takewhile(_split_cond, request_parts)
+                for x in itertools.takewhile(_SPLIT_CONDITION, request_parts)
             ] + [b""]
             request_parts = [x.bytes for x in request_parts]
 
@@ -123,13 +148,13 @@ class Server(object):
         response = command_callable.process_request(json_body, *other_parts)
         if isinstance(response, async.Future):
             response.add_callback(lambda resp:
-                [constants.PROTO_STATUS_SUCCESS, resp])
+                [constants.PROTO_STATUS_SUCCESS] + resp)
             return response
 
-        return [constants.PROTO_STATUS_SUCCESS, response]
+        return [constants.PROTO_STATUS_SUCCESS] + response
 
 
 # Private
 
 
-_split_cond = operator.attrgetter("bytes")
+_SPLIT_CONDITION = operator.attrgetter("bytes")
