@@ -57,6 +57,22 @@ class TestFuture(BaseIntegrationTest, unittest.TestCase):
 
         self._runMe(_handler, _check)
 
+    def testFutureAttachments(self):
+        def _finish(future):
+            future.response(42, attachment=b"Hello")
+
+        def _handler(request):
+            self.assertEqual(request.interface, "test.testFutureAttachments")
+            future = zerojson.Future(request)
+            self.ioloop.add_callback(functools.partial(_finish, future))
+            return future
+
+        def _check():
+            cli = zerojson.Client(self.address, "test.testFutureAttachments")
+            self.assertEqual(cli.invoke("method", args=100500), (42, b"Hello"))
+
+        self._runMe(_handler, _check)
+
     def testFutureIsNow(self):
         def _handler(request):
             self.assertEqual(request.interface, "test.testFutureCall")
