@@ -100,6 +100,18 @@ class FakeClientSocketAttachment(FakeClientSocket):
         return [constants.PROTO_STATUS_SUCCESS, self.data]
 
 
+class FakeClientSocketAttachment2(FakeClientSocket):
+
+    def send_multipart(self, data, flags=0, **kw):
+        assert flags == zmq.NOBLOCK
+        assert len(data) == 2
+        assert data[0] == constants.PROTO_CMD_CALL
+        self.data = data[1]
+
+    def recv_multipart(self, flags=0, **kw):
+        return [constants.PROTO_STATUS_SUCCESS, self.data, b"attachment!"]
+
+
 class FakeClientSocketError(FakeClientSocket):
 
     def __init__(self, error):
@@ -132,7 +144,6 @@ class RemoteComponentTest(unittest.TestCase):
             "interface": "iface.name",
             "method": "method",
             "version": constants.PROTO_VERSION,
-            "session_id": None,
             "args": six.u("\u041f\u0440\u0438\u0432\u0435\u0442")
         })
         self.assertEqual(comp.session_id, None)
@@ -153,6 +164,15 @@ class RemoteComponentTest(unittest.TestCase):
         self.assertEqual(status, constants.PROTO_STATUS_SUCCESS)
         self.assertEqual(message, "42")
 
+    def testSuccessWithAttachment2(self):
+        sock = FakeClientSocketAttachment2()
+        comp = zerojson.Client(sock, "iface.name")
+        status, message, att = comp._send_message(constants.PROTO_CMD_CALL,
+            "42", constants.PROTO_DEFAULT_TIMEOUT)
+        self.assertEqual(status, constants.PROTO_STATUS_SUCCESS)
+        self.assertEqual(message, "42")
+        self.assertEqual(att, b"attachment!")
+
     def testClose(self):
         sock = FakeClientSocket()
         comp = zerojson.Client(sock, "iface.name")
