@@ -81,7 +81,7 @@ class TestCommand(unittest.TestCase):
         class FakeCommand(object):
 
             def process_request(self, json_body, x, y):
-                return json_body, x, y
+                return [json_body, x, y]
 
         self.existing_command = "demo!"
 
@@ -93,7 +93,7 @@ class TestCommand(unittest.TestCase):
     def testKnownCommand(self):
         self.assertEqual(self.server._command(self.existing_command,
             "BODY", "1", "2"),
-            [zerojson.constants.PROTO_STATUS_SUCCESS, ("BODY", "1", "2")])
+            [zerojson.constants.PROTO_STATUS_SUCCESS, "BODY", "1", "2"])
 
     def testUnknownCommand(self):
         self.assertEqual(self.server._command(self.non_existing_command,
@@ -168,6 +168,27 @@ class TestClientServer(BaseIntegrationTest, unittest.TestCase):
 
         self._runMe(_handler, _check)
 
+    def testAttachmentIn(self):
+        def _handler(request):
+            return request.response(request.attachment.decode("utf-8"))
+
+        def _check():
+            cli = zerojson.Client(self.address, "test.testAttachmentIn")
+            self.assertEqual(cli.invoke("method", attachment=b"Hello"),
+                "Hello")
+
+        self._runMe(_handler, _check)
+
+    def testAttachmentOut(self):
+        def _handler(request):
+            return request.response(42, attachment=b"Bye")
+
+        def _check():
+            cli = zerojson.Client(self.address, "test.testAttachmentIn")
+            self.assertEqual(cli.invoke("method"), (42, b"Bye"))
+
+        self._runMe(_handler, _check)
+
     def testSimpleFailure(self):
         def _handler(request):
             raise zerojson.BadRequest("testSimpleFailure")
